/*
 * Copyright © 2016 - 2017 Copyright (c) Sprint, Inc. and others.  All rights reserved.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v1.0 which accompanies this distribution,
 * and is available at http://www.eclipse.org/legal/epl-v10.html
 */
package org.onosproject.fpcagent.util.eventStream;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * SSE Server implementation
 */
public class JettyServer {
	private static final Logger log = LoggerFactory.getLogger(JettyServer.class);

	/**
	 * Method used to initialize and start the SSE server
	 */
	public static void init() {
		Server server = new Server(8070);
		ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
		context.setContextPath("/");

		ServletHolder requestServletHolder = new ServletHolder(EventServer.class);
		context.addServlet(requestServletHolder, "/response");

		ServletHolder notificationServletHolder = new ServletHolder(NotificationServer.class);
		context.addServlet(notificationServletHolder, "/notification");

		server.setHandler(context);
		context.addEventListener(new ConfigureService());
		context.addEventListener(new NotificationService());

		try {
			server.start();
			log.info("Server Started");
			server.join();
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
		}
	}

}
